from decimal import Decimal

from django.db import models


TX_FAILED = -5
TX_RECURRING_CANCELLED = -4
TX_INSUFFICIENT_FUNDS = -3
TX_TIME_OUT = -2
TX_REJECTED = -1
TX_ACCEPTED = 0
TX_PENDING = 1
TX_AUTHORIZED = 2
TX_PAID = 3
TX_VOIDED = 4

TRANSACTION_STATUS_CHOICES = (
    (TX_FAILED, 'Failed'),
    (TX_RECURRING_CANCELLED, 'Recurring Cancelled'),
    (TX_INSUFFICIENT_FUNDS, 'Insufficient Funds'),
    (TX_TIME_OUT, 'Timed Out'),
    (TX_REJECTED, 'Rejected'),
    (TX_ACCEPTED, 'Accepted'),
    (TX_PENDING, 'Pending'),
    (TX_AUTHORIZED, 'Authorized'),
    (TX_PAID, 'Paid'),
    (TX_VOIDED, 'Voided')
)


class MBillsTransaction(models.Model):
    transaction_id = models.CharField(max_length=255)

    amount = models.DecimalField(max_digits=20, decimal_places=2, default=Decimal('0'))
    currency = models.CharField(max_length=255, default='EUR')
    purpose = models.CharField(max_length=255)

    payment_reference = models.CharField(max_length=255, null=True, blank=True)
    capture = models.BooleanField(default=True)

    order_id = models.CharField(max_length=255, null=True, blank=True)
    channel_id = models.CharField(max_length=255, null=True, blank=True)

    payment_token_number = models.CharField(max_length=255)

    status = models.IntegerField(choices=TRANSACTION_STATUS_CHOICES, default=TX_ACCEPTED)

    transaction_type = models.CharField(max_length=255, default='T')
    settled_amount = models.DecimalField(max_digits=20, decimal_places=2, default=Decimal('0'))
    fees = models.DecimalField(max_digits=20, decimal_places=2, default=Decimal('0'))

    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    class Meta:
        verbose_name = "MBills Transaction"

    def __str__(self):
        return self.transaction_id



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-10-11 15:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mbills', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mbillstransaction',
            name='status',
            field=models.IntegerField(choices=[(-5, 'Failed'), (-4, 'Recurring Cancelled'), (-3, 'Insufficient Funds'), (-2, 'Timed Out'), (-1, 'Rejected'), (0, 'Accepted'), (1, 'Pending'), (2, 'Authorized'), (3, 'Paid'), (4, 'Voided')], default=0),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-10-11 08:06
from __future__ import unicode_literals

from decimal import Decimal
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MBillsTransaction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('transaction_id', models.CharField(max_length=255)),
                ('amount', models.DecimalField(decimal_places=2, default=Decimal('0'), max_digits=20)),
                ('currency', models.CharField(default='EUR', max_length=255)),
                ('purpose', models.CharField(max_length=255)),
                ('payment_reference', models.CharField(blank=True, max_length=255, null=True)),
                ('capture', models.BooleanField(default=True)),
                ('order_id', models.CharField(blank=True, max_length=255, null=True)),
                ('channel_id', models.CharField(blank=True, max_length=255, null=True)),
                ('payment_token_number', models.CharField(max_length=255)),
                ('status', models.CharField(choices=[(-5, 'Failed'), (-4, 'Recurring Cancelled'), (-3, 'Insufficient Funds'), (-2, 'Timed Out'), (-1, 'Rejected'), (0, 'Accepted'), (1, 'Pending'), (2, 'Authorized'), (3, 'Paid'), (4, 'Voided')], default=0, max_length=255)),
                ('transaction_type', models.CharField(default='T', max_length=255)),
                ('settled_amount', models.DecimalField(decimal_places=2, default=Decimal('0'), max_digits=20)),
                ('fees', models.DecimalField(decimal_places=2, default=Decimal('0'), max_digits=20)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
            ],
            options={
                'verbose_name': 'MBills Transaction',
            },
        ),
    ]



import warnings

from decimal import Decimal

from django.conf import settings

from python_mbills.api import MBillsAPI
from python_mbills.exceptions import SignatureValidationException

from mbills.models import MBillsTransaction

MBILLS_RSA_PUBLIC_KEY = getattr(settings, 'MBILLS_RSA_PUBLIC_KEY', None)
MBILLS_API_KEY = getattr(settings, 'MBILLS_API_KEY', None)
MBILLS_SECRET_KEY = getattr(settings, 'MBILSS_SECRET_KEY', None)

MBILLS_API_ENDPOINT = getattr(settings, 'MBILSS_API_ENDPOINT', 'https://demo3.halcom.com/MBillsWS')

if MBILLS_API_KEY is None or MBILLS_SECRET_KEY is None:
    warnings.warn("Missing MBILLS_API_KEY and/or MBILLS_SECRET_KEY from settings. API calls will fail!")

if MBILLS_RSA_PUBLIC_KEY is None:
    warnings.warn("Failed to load MBILLS_RSA_PUBLIC_KEY from settings. None of the API calls will be verified!")

# Configure the class only once...better than loading everytime a request is made...
mbills_api = MBillsAPI(api_key=MBILLS_API_KEY, shared_secret=MBILLS_SECRET_KEY,
                       mbills_rsa_pub_key=MBILLS_RSA_PUBLIC_KEY,
                       nonce_length=10, api_endpoint=MBILLS_API_ENDPOINT)


def generate_new_transaction(amount, purpose, payment_reference=None, order_id=None, channel_id=None, capture=True):
    """
    Create a new transaction object. 

    :param amount: 
    :param purpose: 
    :param payment_reference: 
    :param order_id: 
    :param channel_id: 
    :param capture: 
    :return: 
    """
    tx_id, token_number, status = mbills_api.create_new_sale(amount, purpose, payment_reference, order_id, channel_id,
                                                             capture)

    mbills_transaction = MBillsTransaction(transaction_id=tx_id,
                                           amount=amount,
                                           currency=mbills_api.currency,
                                           purpose=purpose,
                                           payment_reference=payment_reference,
                                           capture=capture,
                                           order_id=order_id,
                                           channel_id=channel_id,
                                           payment_token_number=token_number,
                                           status=status,
                                           )
    mbills_transaction.save()

    return mbills_transaction


def capture_transaction(transaction, capture_amount, message=''):
    """
    Capture existing transaction. 
    :param transaction: 
    :param capture_amount: 
    :param message: 
    :return: 
    """
    status_code = mbills_api.capture_sale(transaction.transaction_id, capture_amount=capture_amount, message=message)

    transaction.status = status_code
    transaction.save()

    return transaction


def void_transaction(transaction, message=''):
    """
    Void transaction.
    :param transaction: 
    :param message: 
    :return: 
    """
    status_code = mbills_api.void_preauthorization(transaction.transaction_id, message=message)

    transaction.status = status_code
    transaction.save()

    return transaction


def update_transaction_status(transaction):
    """
    Update the transaction status and return the refreshed transcation object
    :param transaction: 
    :return: 
    """
    tx_json = mbills_api.fetch_transaction_status(transaction_id=transaction.transaction_id)

    return _update_transaction_from_data(transaction, tx_json)


def handle_webhook(json_data):
    """
    Fetch the transaction from the database and update it's status.

    If the transaction does not exist in the database this function will 
    raise a DoesNotExist exception on the MBillsTransaction object.

    If the received webhook data server signature is not valid it will raise SignatureValidationException exception.

    :param json_data: data received from the webhook
    :return: 
    """
    if not mbills_api.base.verify_response(json_data):
        raise SignatureValidationException('Server signature verification has failed')

    transaction = MBillsTransaction.objects.get(transaction_id=json_data.get('transactionid'))

    return _update_transaction_from_data(transaction, json_data)


def _update_transaction_from_data(transaction, json_data):
    # Update the fields...
    transaction.status = json_data.get('status', transaction.status)
    transaction.settled_amount = Decimal(json_data.get('settledamount', 0) / 100.0)
    transaction.fees = Decimal(json_data.get('fees', 0) / 100.0)
    transaction.transaction_type = json_data.get('transactiontype', transaction.transaction_type)

    transaction.save()

    return transaction



from distutils.core import setup

setup(
  name='django-mbills',
  packages=['mbills', 'mbills.migrations'],
  version='0.1.9',
  description='Django implementation of the Hal MBills APIs.',
  author='Boris Savic',
  author_email='boris70@gmail.com',
  url='https://github.com/boris-savic/django-mbills',
  keywords=['mbills', 'halcom', 'django' ],
  classifiers=[],
  install_requires=[
        'python_mbills>=0.1.3',
    ]
)